==> zego_zimkit-1.13.3.tar.gz: lib/src/components/message_list.dart <==
import 'dart:async';

import 'package:flutter/material.dart';

import 'package:zego_zim/zego_zim.dart';

import 'package:zego_zimkit/src/components/components.dart';
import 'package:zego_zimkit/src/components/defines.dart';
import 'package:zego_zimkit/src/services/services.dart';

// featureList
class ZIMKitMessageListView extends StatefulWidget {
  const ZIMKitMessageListView({
    Key? key,
    required this.conversationID,
    this.conversationType = ZIMConversationType.peer,
    this.onPressed,
    this.itemBuilder,
    this.backgroundBuilder,
    this.loadingBuilder,
    this.onLongPress,
    this.errorBuilder,
    this.scrollController,
    this.theme,
  }) : super(key: key);

  final String conversationID;
  final ZIMConversationType conversationType;

  final ScrollController? scrollController;

  final void Function(
    BuildContext context,
    ZIMKitMessage message,
    Function defaultAction,
  )? onPressed;
  final void Function(
    BuildContext context,
    LongPressStartDetails details,
    ZIMKitMessage message,
    Function defaultAction,
  )? onLongPress;
  final Widget Function(
    BuildContext context,
    ZIMKitMessage message,
    Widget defaultWidget,
  )? itemBuilder;
  final Widget Function(
    BuildContext context,
    Widget defaultWidget,
  )? errorBuilder;
  final Widget Function(
    BuildContext context,
    Widget defaultWidget,
  )? loadingBuilder;
  final Widget Function(
    BuildContext context,
    Widget defaultWidget,
  )? backgroundBuilder;

  // theme
  final ThemeData? theme;

  @override
  State<ZIMKitMessageListView> createState() => _ZIMKitMessageListViewState();
}

class _ZIMKitMessageListViewState extends State<ZIMKitMessageListView> {
  final ScrollController _defaultScrollController = ScrollController();

  ScrollController get _scrollController =>
      widget.scrollController ?? _defaultScrollController;

  Completer? _loadMoreCompleter;

  @override
  void initState() {
    ZIMKit().clearUnreadCount(widget.conversationID, widget.conversationType);
    _scrollController.addListener(scrollControllerListener);

    super.initState();
  }

  @override
  void dispose() {
    ZIMKit().clearUnreadCount(widget.conversationID, widget.conversationType);
    _scrollController.removeListener(scrollControllerListener);

    super.dispose();
  }

  Future<void> scrollControllerListener() async {
    if (_loadMoreCompleter == null || _loadMoreCompleter!.isCompleted) {
      if (_scrollController.position.pixels >=
          0.8 * _scrollController.position.maxScrollExtent) {
        _loadMoreCompleter = Completer();
        if (0 ==
            await ZIMKit().loadMoreMessage(
                widget.conversationID, widget.conversationType)) {
          _scrollController.removeListener(scrollControllerListener);
        }
        _loadMoreCompleter!.complete();
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Theme(
      data: widget.theme ?? Theme.of(context),
      child: Expanded(
        child: FutureBuilder(
          future: ZIMKit().getMessageListNotifier(
            widget.conversationID,
            widget.conversationType,
          ),
          builder: (context, snapshot) {
            if (snapshot.hasData) {
              return ValueListenableBuilder(
                valueListenable: snapshot.data! as ZIMKitMessageListNotifier,
                builder: (
                  BuildContext context,
                  List<ValueNotifier<ZIMKitMessage>> messageList,
                  Widget? child,
                ) {
                  ZIMKit().clearUnreadCount(
                    widget.conversationID,
                    widget.conversationType,
                  );

                  return listview(
                    messageList: messageList,
                  );
                },
              );
            } else if (snapshot.hasError) {
              // TODO 未实现加载失败
              // defaultWidget
              final Widget defaultWidget = Center(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    IconButton(
                      onPressed: () => setState(() {}),
                      icon: const Icon(Icons.refresh_rounded),
                    ),
                    Text(snapshot.error.toString()),
                    const Text('Load failed, please click to retry'),
                  ],
                ),
              );

              // customWidget
              return GestureDetector(
                onTap: () => setState(() {}),
                child: widget.errorBuilder?.call(context, defaultWidget) ??
                    defaultWidget,
              );
            } else {
              // defaultWidget
              const Widget defaultWidget =
                  Center(child: CircularProgressIndicator());

              // customWidget
              return widget.loadingBuilder?.call(context, defaultWidget) ??
                  defaultWidget;
            }
          },
        ),
      ),
    );
  }

  Widget listview({
    required List<ValueNotifier<ZIMKitMessage>> messageList,
  }) {
    return LayoutBuilder(builder: (context, BoxConstraints constraints) {
      return Stack(
        children: [
          SizedBox(
            height: constraints.maxHeight,
            width: constraints.maxWidth,
            child: widget.backgroundBuilder
                    ?.call(context, const SizedBox.shrink()) ??
                const SizedBox.shrink(),
          ),
          ListView.builder(
            cacheExtent: constraints.maxHeight * 3,
            reverse: true,
            controller: _scrollController,
            itemCount: messageList.length,
            itemBuilder: (context, index) {
              final reversedIndex = messageList.length - index - 1;
              final messageNotifier = messageList[reversedIndex];

              return ValueListenableBuilder(
                valueListenable: messageNotifier,
                builder: (
                  BuildContext context,
                  ZIMKitMessage message,
                  Widget? child,
                ) {
                  // defaultWidget
                  final defaultWidget = defaultListItem(
                    message: message,
                    constraints: constraints,
                  );
                  // TODO spacing
                  // TODO 时间间隔
                  // customWidget
                  return widget.itemBuilder?.call(
                        context,
                        message,
                        defaultWidget,
                      ) ??
                      defaultWidget;
                },
              );
            },
          ),
        ],
      );
    });
  }

  Widget defaultListItem({
    required ZIMKitMessage message,
    required BoxConstraints constraints,
  }) {
    // defaultWidget
    /// only other user in group
    final isDisplayNickName =
        ZIMConversationType.group == widget.conversationType && !message.isMine;
    final defaultNickName = Positioned(
      top: nickNameHeight / 2,
      left: message.isMine
          ? null
          : itemHorizontalMargin + avatarWidth + messagePointerWidth,
      right: message.isMine
          ? itemHorizontalMargin + avatarWidth + messagePointerWidth
          : null,
      child: FutureBuilder(
        future: ZIMKit().queryUser(
          message.info.senderUserID,
        ),
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            final userInfo = snapshot.data! as ZIMUserFullInfo;
            return Text(
              userInfo.baseInfo.userName,
              textAlign: TextAlign.left,
              style: TextStyle(
                color: Theme.of(context)
                    .textTheme
                    .bodyLarge!
                    .color
                    ?.withOpacity(0.6),
              ),
            );
          }

          return Container();
        },
      ),
    );

    final defaultMessageItem = Container(
      width: constraints.maxWidth,

      /// add top padding for display nickname
      padding: EdgeInsets.only(
        top: isDisplayNickName ? nickNameHeight * 1.5 : 0,
      ),
      child: ConstrainedBox(
        constraints: BoxConstraints(
          maxWidth: constraints.maxWidth,
          maxHeight: message.type == ZIMMessageType.text
              ? double.maxFinite
              : constraints.maxHeight * 0.5,
        ),
        child: ZIMKitMessageWidget(
          key: ValueKey(message.hashCode),
          message: message,
          onPressed: widget.onPressed,
          onLongPress: widget.onLongPress,
        ),
      ),
    );

    final Widget defaultWidget = Stack(
      children: [
        if (isDisplayNickName) defaultNickName else Container(),
        defaultMessageItem,
      ],
    );

    // TODO spacing
    // TODO 时间间隔
    // customWidget
    return widget.itemBuilder?.call(
          context,
          message,
          defaultWidget,
        ) ??
        defaultWidget;
  }
}

==> zego_zimkit-1.17.0.tar.gz: lib/src/components/message_list.dart <==
import 'dart:async';

import 'package:flutter/material.dart';
import 'package:zego_zim/zego_zim.dart';
import 'package:zego_zimkit/src/components/components.dart';
import 'package:zego_zimkit/src/services/services.dart';

// featureList
class ZIMKitMessageListView extends StatefulWidget {
  const ZIMKitMessageListView({
    Key? key,
    required this.conversationID,
    this.conversationType = ZIMConversationType.peer,
    this.onPressed,
    this.itemBuilder,
    this.messageContentBuilder,
    this.backgroundBuilder,
    this.loadingBuilder,
    this.onLongPress,
    this.errorBuilder,
    this.scrollController,
    this.theme,
  }) : super(key: key);

  final String conversationID;
  final ZIMConversationType conversationType;

  final ScrollController? scrollController;

  final void Function(BuildContext context, ZIMKitMessage message, Function defaultAction)? onPressed;
  final void Function(
      BuildContext context, LongPressStartDetails details, ZIMKitMessage message, Function defaultAction)? onLongPress;
  final Widget Function(BuildContext context, ZIMKitMessage message, Widget defaultWidget)? itemBuilder;
  final Widget Function(BuildContext context, ZIMKitMessage message, Widget defaultWidget)? messageContentBuilder;
  final Widget Function(BuildContext context, Widget defaultWidget)? errorBuilder;
  final Widget Function(BuildContext context, Widget defaultWidget)? loadingBuilder;
  final Widget Function(BuildContext context, Widget defaultWidget)? backgroundBuilder;

  // theme
  final ThemeData? theme;

  @override
  State<ZIMKitMessageListView> createState() => _ZIMKitMessageListViewState();
}

class _ZIMKitMessageListViewState extends State<ZIMKitMessageListView> {
  final ScrollController _defaultScrollController = ScrollController();

  ScrollController get _scrollController => widget.scrollController ?? _defaultScrollController;

  Completer? _loadMoreCompleter;

  @override
  void initState() {
    ZIMKit().clearUnreadCount(widget.conversationID, widget.conversationType);
    _scrollController.addListener(scrollControllerListener);

    super.initState();
  }

  @override
  void dispose() {
    ZIMKit().clearUnreadCount(widget.conversationID, widget.conversationType);
    _scrollController.removeListener(scrollControllerListener);

    super.dispose();
  }

  Future<void> scrollControllerListener() async {
    if (_loadMoreCompleter == null || _loadMoreCompleter!.isCompleted) {
      if (_scrollController.position.pixels >= 0.8 * _scrollController.position.maxScrollExtent) {
        _loadMoreCompleter = Completer();
        if (0 == await ZIMKit().loadMoreMessage(widget.conversationID, widget.conversationType)) {
          _scrollController.removeListener(scrollControllerListener);
        }
        _loadMoreCompleter!.complete();
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Theme(
      data: widget.theme ?? Theme.of(context),
      child: Expanded(
        child: FutureBuilder(
          future: ZIMKit().getMessageListNotifier(
            widget.conversationID,
            widget.conversationType,
          ),
          builder: (context, snapshot) {
            if (snapshot.hasData) {
              return ValueListenableBuilder(
                valueListenable: snapshot.data! as ZIMKitMessageListNotifier,
                builder: (
                  BuildContext context,
                  List<ValueNotifier<ZIMKitMessage>> messageList,
                  Widget? child,
                ) {
                  ZIMKit().clearUnreadCount(
                    widget.conversationID,
                    widget.conversationType,
                  );

                  return listview(
                    messageList: messageList,
                  );
                },
              );
            } else if (snapshot.hasError) {
              // TODO 未实现加载失败
              // defaultWidget
              final Widget defaultWidget = Center(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    IconButton(
                      onPressed: () => setState(() {}),
                      icon: const Icon(Icons.refresh_rounded),
                    ),
                    Text(snapshot.error.toString()),
                    const Text('Load failed, please click to retry'),
                  ],
                ),
              );

              // customWidget
              return GestureDetector(
                onTap: () => setState(() {}),
                child: widget.errorBuilder?.call(context, defaultWidget) ?? defaultWidget,
              );
            } else {
              // defaultWidget
              const Widget defaultWidget = Center(child: CircularProgressIndicator());

              // customWidget
              return widget.loadingBuilder?.call(context, defaultWidget) ?? defaultWidget;
            }
          },
        ),
      ),
    );
  }

  Widget listview({
    required List<ValueNotifier<ZIMKitMessage>> messageList,
  }) {
    return LayoutBuilder(builder: (context, BoxConstraints constraints) {
      return Stack(
        children: [
          SizedBox(
            height: constraints.maxHeight,
            width: constraints.maxWidth,
            child: widget.backgroundBuilder?.call(context, const SizedBox.shrink()) ?? const SizedBox.shrink(),
          ),
          ListView.builder(
            cacheExtent: constraints.maxHeight * 3,
            reverse: true,
            controller: _scrollController,
            itemCount: messageList.length,
            itemBuilder: (context, index) {
              final reversedIndex = messageList.length - index - 1;
              final messageNotifier = messageList[reversedIndex];

              return ValueListenableBuilder(
                valueListenable: messageNotifier,
                builder: (
                  BuildContext context,
                  ZIMKitMessage message,
                  Widget? child,
                ) {
                  // defaultWidget
                  final defaultWidget = defaultMessageWidget(
                    message: message,
                    constraints: constraints,
                  );
                  // TODO spacing
                  // TODO 时间间隔
                  return widget.itemBuilder?.call(
                        context,
                        message,
                        defaultWidget,
                      ) ??
                      defaultWidget;
                },
              );
            },
          ),
        ],
      );
    });
  }

  Widget defaultMessageWidget({required ZIMKitMessage message, required BoxConstraints constraints}) {
    return SizedBox(
      width: constraints.maxWidth,
      child: ConstrainedBox(
        constraints: BoxConstraints(
          maxWidth: constraints.maxWidth,
          maxHeight: message.type == ZIMMessageType.text ? double.maxFinite : constraints.maxHeight * 0.5,
        ),
        child: ZIMKitMessageWidget(
          key: ValueKey(message.hashCode),
          message: message,
          onPressed: widget.onPressed,
          onLongPress: widget.onLongPress,
          messageContentBuilder: widget.messageContentBuilder,
        ),
      ),
    );
  }
}
